; Form
; A collection of objects.
;
; Functions:
; 1. guiDrawForm			Either draws all objects in the collection or clears screen, based on FO_Showing.
; 2. guiShowForm			Sets FO_Showing, then calls guiDrawForm
; 3. guiHideForm			Resets FO_Showing, then calls guiDrawForm
; 4. guiHandleClick		Handles a mouse click on the form.
;
; Comments:
;

FO_Showing		equ	0
FO_Inverse		equ	1

guiShowForm:
 inc hl
 set FO_Showing,(hl)
 dec hl
 jr guiDrawForm
guiHideForm:
 inc hl
 res FO_Showing,(hl)
 dec hl

guiDrawForm:
 res plotloc,(iy+plotflags)
 ld b,(hl)
 inc hl
 push bc
 call guiEraseForm
 pop bc
 bit FO_Showing,(hl)
 ret z
 ld de,5
 add hl,de
guiDrawFormLoop:
 push bc
 ld e,(hl)
 inc hl
 ld d,0
 ld ix,guiDrawRoutines
 add ix,de
 ld a,(hl)
 inc hl
 push hl
 ld h,(hl)
 ld l,a
 ld bc,guiDrawFormReturn
 push bc
 jp (ix)
guiDrawFormReturn:
 pop hl
 inc hl
 pop bc
 djnz guiDrawFormLoop
 ret

guiEraseForm:
 push hl
 ld a,(hl)
 inc hl
 ld e,(hl)
 inc hl
 ld d,(hl)
 inc hl
 ld c,(hl)
 inc hl
 ld b,(hl)
 ex de,hl
 push hl
 add hl,bc
 ex de,hl
 pop hl
 bit FO_Inverse,a
 jr nz,guiEraseFormFill
 B_CALL _clearrect
 pop hl
 ret
guiEraseFormFill:
 B_CALL _fillrect
 pop hl
 ret


guiDrawRoutines:		; Hey look! It's polymorphism!
 jp guiDrawForm
 jp guiDrawLabel
 jp guiDrawPara
 jp guiDrawButton
 jp guiDrawCheckBox
 jp guiDrawOptions
 jp guiDrawTextBox
 jp guiDrawBitmap
 jp guiDrawList
 jp guiDrawDrop



; Inputs: hl -> Form
;         de =  Mouse Coords
; Outputs: hl -> Object that was clicked (except if it was a paragraph)
;          ix -> Location in form where object lies (Object Type byte)
;          a  =  Index of object on form (first obj = 0)
;          carry flag set if object not found, reset if object was found (except paragraph)
; Preserved: de
; Comments:
;  Will not find paragraphs. Basically, the paragraph detection routine would be
;   too hard to write, and I couldn't think of a use for clicking on paragraphs,
;   so I didn't bother. If you want to write one yourself, the mouse coordinates
;   will be in register de on return, and you can handle it then.
;
guiHandleClick:
 inc hl
 bit FO_Showing,(hl)
 ret z
 inc hl
 ld b,(hl)
 inc hl
 ld c,(hl)
 inc hl
 ld a,(hl)
 inc hl
 push hl
 pop ix
 inc ix
 ld h,(hl)
 ld l,a
 add hl,bc		; h=maxrow,l=maxcol,b=minrow,c=mincol,d=mouserow,e=mousecol
 call IsInRectangle
 ret c
 ld b,(ix-6)
 ld c,b
guiHandleClickLoop:
 push bc
 ld c,(ix)
 ld b,0
 inc ix
 ld l,(ix)
 inc ix
 ld h,(ix)
 inc ix
 push ix
 ld ix,guiHandleRoutines
 add ix,bc
 ld bc,guiHandleClickRet
 push de
 push bc
 jp (ix)
guiHandleClickRet:
 pop de
 pop ix
 pop bc
 djnz guiHandleClickLoop
 scf
 ret
guiHandleClickFinished:
 pop de		; Get ret location off stack
 pop de		; de = mouse coords
 pop ix		; ix -> Type byte of next thing
 dec ix
 ld h,(ix)
 dec ix
 ld l,(ix)	; hl -> Object
 dec ix		; ix -> Type byte of selected object
 pop bc		; b = items left on form, c = total items
 ld a,c
 sub b
 or a
 ret

guiHandleRoutines:
 jp guiHandleClick		; Nested Forms
 jp guiHandleLabel
 jp guiHandlePara
 jp guiHandleButton
 jp guiHandleCheckBox
 jp guiHandleOptions
 jp guiHandleTextBox
 jp guiHandleBitmap
 jp guiHandleList
 jp guiHandleDrop

; Checks if (e,d) lies within the rectangle bounded by (c,b) on the upper left and (l,h) on the lower right
; Returns nc if inside, c if outside
; Destroys a
IsInRectangle:
 ld a,h
 cp d
 ret c
 ld a,d
 cp b
 ret c
 ld a,l
 cp e
 ret c
 ld a,e
 cp c
 ret
